import zipObject from 'lodash/zipObject';
import _ from 'lodash';
import groupBy from 'lodash/groupBy';
import sortBy from 'lodash/sortBy';
import toPairs from 'lodash/toPairs';
import difference from 'lodash/difference';
import range from 'lodash/range';
import times from 'lodash/times';
import sum from 'lodash/sum';

var VotingSystem;

(function (VotingSystem) {
  VotingSystem["Approbation"] = "APPROBATION";
  VotingSystem["AbsoluteMajority"] = "ABSOLUTE_MAJORITY";
  VotingSystem["Baldwin"] = "BALDWIN";
  VotingSystem["Borda"] = "BORDA";
  VotingSystem["BottomTwoRunoff"] = "BOTTOM_TWO_RUNOFF";
  VotingSystem["Coombs"] = "COOMBS";
  VotingSystem["Copeland"] = "COPELAND";
  VotingSystem["FirstPastThePost"] = "FIRST_PAST_THE_POST";
  VotingSystem["Kemeny"] = "KEMENY";
  VotingSystem["InstantRunoff"] = "INSTANT_RUNOFF";
  VotingSystem["MajorityJudgment"] = "MAJORITY_JUDGEMENT";
  VotingSystem["MaximalLotteries"] = "MAXIMAL_LOTTERIES";
  VotingSystem["Minimax"] = "MINIMAX";
  VotingSystem["Nanson"] = "NANSON";
  VotingSystem["RandomizedCondorcet"] = "RANDOMIZED_CONDORCET";
  VotingSystem["RandomCandidates"] = "RANDOM_CANDIDATES";
  VotingSystem["RandomDictator"] = "RANDOM_DICTATOR";
  VotingSystem["RankedPairs"] = "RANKED_PAIRS";
  VotingSystem["Schulze"] = "SCHULZE";
  VotingSystem["TwoRoundRunoff"] = "TWO_ROUND_RUNOFF";
})(VotingSystem || (VotingSystem = {}));

function _classCallCheck(instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
}

function _defineProperties(target, props) {
  for (var i = 0; i < props.length; i++) {
    var descriptor = props[i];
    descriptor.enumerable = descriptor.enumerable || false;
    descriptor.configurable = true;
    if ("value" in descriptor) descriptor.writable = true;
    Object.defineProperty(target, descriptor.key, descriptor);
  }
}

function _createClass(Constructor, protoProps, staticProps) {
  if (protoProps) _defineProperties(Constructor.prototype, protoProps);
  if (staticProps) _defineProperties(Constructor, staticProps);
  Object.defineProperty(Constructor, "prototype", {
    writable: false
  });
  return Constructor;
}

function _setPrototypeOf(o, p) {
  _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) {
    o.__proto__ = p;
    return o;
  };

  return _setPrototypeOf(o, p);
}

function _inherits(subClass, superClass) {
  if (typeof superClass !== "function" && superClass !== null) {
    throw new TypeError("Super expression must either be null or a function");
  }

  subClass.prototype = Object.create(superClass && superClass.prototype, {
    constructor: {
      value: subClass,
      writable: true,
      configurable: true
    }
  });
  Object.defineProperty(subClass, "prototype", {
    writable: false
  });
  if (superClass) _setPrototypeOf(subClass, superClass);
}

function _getPrototypeOf(o) {
  _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) {
    return o.__proto__ || Object.getPrototypeOf(o);
  };
  return _getPrototypeOf(o);
}

function _isNativeReflectConstruct() {
  if (typeof Reflect === "undefined" || !Reflect.construct) return false;
  if (Reflect.construct.sham) return false;
  if (typeof Proxy === "function") return true;

  try {
    Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {}));
    return true;
  } catch (e) {
    return false;
  }
}

function _assertThisInitialized(self) {
  if (self === void 0) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return self;
}

function _possibleConstructorReturn(self, call) {
  if (call && (typeof call === "object" || typeof call === "function")) {
    return call;
  } else if (call !== void 0) {
    throw new TypeError("Derived constructors may only return object or undefined");
  }

  return _assertThisInitialized(self);
}

function _createSuper(Derived) {
  var hasNativeReflectConstruct = _isNativeReflectConstruct();
  return function _createSuperInternal() {
    var Super = _getPrototypeOf(Derived),
        result;

    if (hasNativeReflectConstruct) {
      var NewTarget = _getPrototypeOf(this).constructor;
      result = Reflect.construct(Super, arguments, NewTarget);
    } else {
      result = Super.apply(this, arguments);
    }

    return _possibleConstructorReturn(this, result);
  };
}

function _arrayLikeToArray(arr, len) {
  if (len == null || len > arr.length) len = arr.length;

  for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i];

  return arr2;
}

function _unsupportedIterableToArray(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(o);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
}

function _createForOfIteratorHelper(o, allowArrayLike) {
  var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"];

  if (!it) {
    if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") {
      if (it) o = it;
      var i = 0;

      var F = function () {};

      return {
        s: F,
        n: function () {
          if (i >= o.length) return {
            done: true
          };
          return {
            done: false,
            value: o[i++]
          };
        },
        e: function (e) {
          throw e;
        },
        f: F
      };
    }

    throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
  }

  var normalCompletion = true,
      didErr = false,
      err;
  return {
    s: function () {
      it = it.call(o);
    },
    n: function () {
      var step = it.next();
      normalCompletion = step.done;
      return step;
    },
    e: function (e) {
      didErr = true;
      err = e;
    },
    f: function () {
      try {
        if (!normalCompletion && it.return != null) it.return();
      } finally {
        if (didErr) throw err;
      }
    }
  };
}

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function ownKeys(object, enumerableOnly) {
  var keys = Object.keys(object);

  if (Object.getOwnPropertySymbols) {
    var symbols = Object.getOwnPropertySymbols(object);
    enumerableOnly && (symbols = symbols.filter(function (sym) {
      return Object.getOwnPropertyDescriptor(object, sym).enumerable;
    })), keys.push.apply(keys, symbols);
  }

  return keys;
}

function _objectSpread2(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = null != arguments[i] ? arguments[i] : {};
    i % 2 ? ownKeys(Object(source), !0).forEach(function (key) {
      _defineProperty(target, key, source[key]);
    }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) {
      Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
    });
  }

  return target;
}

function _arrayWithoutHoles(arr) {
  if (Array.isArray(arr)) return _arrayLikeToArray(arr);
}

function _iterableToArray(iter) {
  if (typeof Symbol !== "undefined" && iter[Symbol.iterator] != null || iter["@@iterator"] != null) return Array.from(iter);
}

function _nonIterableSpread() {
  throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}

function _toConsumableArray(arr) {
  return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread();
}

var isBallotEqual = function isBallotEqual(a, b) {
  if (a.length !== b.length) return false;
  if (a.length === 0) return true;
  return _.every(a, function (rank, k) {
    return _.isEqual(_toConsumableArray(rank).sort(), _toConsumableArray(b[k]).sort());
  });
};
var normalizeRankInput = function normalizeRankInput(rankinput) {
  return rankinput.map(function (rank) {
    return typeof rank === 'string' ? [rank] : rank;
  });
};
var groupBallots = function groupBallots(ballots) {
  return ballots.reduce(function (acc, ballot) {
    var match = acc.findIndex(function (b) {
      return isBallotEqual(b.ranking, ballot.ranking);
    });
    if (match === -1) return [].concat(_toConsumableArray(acc), [ballot]);
    return acc.map(function (b, k) {
      return k === match ? _objectSpread2(_objectSpread2({}, b), {}, {
        weight: b.weight + ballot.weight
      }) : b;
    });
  }, []).filter(function (b) {
    return b.weight !== 0;
  }).sort(function (a, b) {
    return b.weight - a.weight;
  });
};
var toWeightedBallots = function toWeightedBallots(ballots) {
  return ballots.reduce(function (w, ballot) {
    var match = w.findIndex(function (ww) {
      return isBallotEqual(ww.ranking, ballot);
    });

    if (match === -1) {
      return [].concat(_toConsumableArray(w), [{
        ranking: ballot,
        weight: 1
      }]);
    }

    return w.map(function (ww, k) {
      return k === match ? {
        ranking: w[match].ranking,
        weight: w[match].weight + 1
      } : ww;
    });
  }, []);
};
var checkDuplicatedCandidate = function checkDuplicatedCandidate(ranking) {
  ranking.reduce(function (acc, rank) {
    var inter = _.intersection(acc, rank);

    if (inter.length > 0) {
      throw new Error("Some candidates are present multiple times: ".concat(inter));
    }

    return [].concat(_toConsumableArray(acc), _toConsumableArray(rank));
  }, []);
};
var removeDuplicatedCandidates = function removeDuplicatedCandidates(ranking) {
  return ranking.reduce(function (acc, cur) {
    var unique = _.difference(_.uniq(cur), acc.usedCandidates);

    return {
      ranking: unique.length > 0 ? [].concat(_toConsumableArray(acc.ranking), [unique]) : acc.ranking,
      usedCandidates: [].concat(_toConsumableArray(acc.usedCandidates), _toConsumableArray(unique))
    };
  }, {
    ranking: [],
    usedCandidates: []
  }).ranking;
};
var removeInvalidCandidates = function removeInvalidCandidates(ranking, candidates) {
  return ranking.map(function (names) {
    return names.filter(function (name) {
      return candidates.includes(name);
    });
  }).filter(function (rank) {
    return rank.length;
  });
};
var normalizeRanking = function normalizeRanking(ranking, candidates) {
  return removeDuplicatedCandidates(removeInvalidCandidates(ranking, candidates));
};
var normalizeBallot = function normalizeBallot(ballot, candidates) {
  return _objectSpread2(_objectSpread2({}, ballot), {}, {
    ranking: normalizeRanking(ballot.ranking, candidates)
  });
};
var candidatesFromBallots = function candidatesFromBallots(ballots) {
  return ballots.reduce(function (acc, curr) {
    return _.uniq([].concat(_toConsumableArray(acc), _toConsumableArray(curr.ranking.flat())));
  }, []);
};
var normalizeBallots = function normalizeBallots(ballots, candidates) {
  var c = candidates || candidatesFromBallots(ballots);
  return ballots.map(function (ballot) {
    return normalizeBallot(ballot, c);
  });
};
var totalBallotsWeight = function totalBallotsWeight(ballots) {
  return ballots.reduce(function (acc, ballot) {
    return acc + ballot.weight;
  }, 0);
};

var matrixFromBallots = function matrixFromBallots(ballots, candidates) {
  var array = times(candidates.length, function () {
    return times(candidates.length, function () {
      return 0;
    });
  });

  var _iterator = _createForOfIteratorHelper(ballots),
      _step;

  try {
    for (_iterator.s(); !(_step = _iterator.n()).done;) {
      var ranking = _step.value;
      var rIndex = ranking.ranking.map(function (rank) {
        return rank.map(function (c) {
          return candidates.indexOf(c);
        });
      });
      var rankedLower = range(candidates.length);

      var _iterator2 = _createForOfIteratorHelper(rIndex),
          _step2;

      try {
        for (_iterator2.s(); !(_step2 = _iterator2.n()).done;) {
          var rank = _step2.value;
          rankedLower = difference(rankedLower, rank);

          var _iterator3 = _createForOfIteratorHelper(rank),
              _step3;

          try {
            for (_iterator3.s(); !(_step3 = _iterator3.n()).done;) {
              var w = _step3.value;

              var _iterator4 = _createForOfIteratorHelper(rankedLower),
                  _step4;

              try {
                for (_iterator4.s(); !(_step4 = _iterator4.n()).done;) {
                  var l = _step4.value;
                  array[w][l] += ranking.weight;
                }
              } catch (err) {
                _iterator4.e(err);
              } finally {
                _iterator4.f();
              }
            }
          } catch (err) {
            _iterator3.e(err);
          } finally {
            _iterator3.f();
          }
        }
      } catch (err) {
        _iterator2.e(err);
      } finally {
        _iterator2.f();
      }
    }
  } catch (err) {
    _iterator.e(err);
  } finally {
    _iterator.f();
  }

  return {
    array: array,
    candidates: candidates
  };
};
var makeAntisymetric = function makeAntisymetric(matrix) {
  return {
    array: matrix.array.map(function (values, row) {
      return values.map(function (v, col) {
        return v - matrix.array[col][row];
      });
    }),
    candidates: matrix.candidates
  };
};

var scoresToRanking = function scoresToRanking(scores) {
  return sortBy(groupBy(toPairs(scores), function (e) {
    return e[1];
  }), function (e) {
    return -e[0][1];
  }).map(function (e) {
    return e.map(function (f) {
      return f[0];
    });
  });
};

// From chadhutchins:
// https://gist.github.com/chadhutchins/1440602
var Vertex = /*#__PURE__*/function () {
  function Vertex(name) {
    _classCallCheck(this, Vertex);

    this.connections = [];
    this.name = name; // used in tarjan algorithm
    // went ahead and explicity initalized them

    this.index = -1;
    this.lowlink = -1;
  }

  _createClass(Vertex, [{
    key: "equals",
    value: function equals(vertex) {
      // equality check based on vertex name
      return this.name === vertex.name;
    }
  }, {
    key: "connect",
    value: function connect(vertex) {
      this.connections.push(vertex);
    }
  }]);

  return Vertex;
}();

// From chadhutchins:
// https://gist.github.com/chadhutchins/1440602
var VertexStack = /*#__PURE__*/function () {
  function VertexStack() {
    _classCallCheck(this, VertexStack);

    this.vertices = [];
  }

  _createClass(VertexStack, [{
    key: "contains",
    value: function contains(vertex) {
      for (var i in this.vertices) {
        if (this.vertices[i].equals(vertex)) return true;
      }

      return false;
    }
  }]);

  return VertexStack;
}();

var Tarjan = /*#__PURE__*/function () {
  function Tarjan(graph) {
    _classCallCheck(this, Tarjan);

    this.index = 0;
    this.stack = new VertexStack();
    this.graph = graph;
    this.scc = [];
  }

  _createClass(Tarjan, [{
    key: "run",
    value: function run() {
      for (var i in this.graph) {
        if (this.graph[i].index < 0) this.strongconnect(this.graph[i]);
      }

      return this.scc;
    }
  }, {
    key: "strongconnect",
    value: function strongconnect(vertex) {
      // Set the depth index for v to the smallest unused index
      vertex.index = this.index;
      vertex.lowlink = this.index;
      this.index = this.index + 1;
      this.stack.vertices.push(vertex); // Consider successors of v
      // aka... consider each vertex in vertex.connections

      var _iterator = _createForOfIteratorHelper(vertex.connections),
          _step;

      try {
        for (_iterator.s(); !(_step = _iterator.n()).done;) {
          var connection = _step.value;
          var v = vertex;
          var _w = connection;

          if (_w.index < 0) {
            // Successor w has not yet been visited; recurse on it
            this.strongconnect(_w);
            v.lowlink = Math.min(v.lowlink, _w.lowlink);
          } else if (this.stack.contains(_w)) {
            // Successor w is in stack S and hence in the current SCC
            v.lowlink = Math.min(v.lowlink, _w.index);
          }
        } // If v is a root node, pop the stack and generate an SCC

      } catch (err) {
        _iterator.e(err);
      } finally {
        _iterator.f();
      }

      if (vertex.lowlink === vertex.index) {
        // start a new strongly connected component
        var vertices = [];
        var w = null;
        if (this.stack.vertices.length > 0) do {
          w = this.stack.vertices.pop(); // add w to current strongly connected component

          if (w) vertices.push(w);
        } while (w && !vertex.equals(w)); // output the current strongly connected component
        // ... i'm going to push the results to a member scc array variable

        if (vertices.length > 0) this.scc.push(vertices);
      }
    }
  }]);

  return Tarjan;
}();

var generateAcyclicGraph = function generateAcyclicGraph(graph, edgesToAdd) {
  var allEdges = [].concat(_toConsumableArray(graph), _toConsumableArray(edgesToAdd));
  var vDict = {};

  var _iterator = _createForOfIteratorHelper(_.uniq(allEdges.flatMap(function (e) {
    return [e.from, e.to];
  }))),
      _step;

  try {
    for (_iterator.s(); !(_step = _iterator.n()).done;) {
      var c = _step.value;
      vDict[c] = new Vertex(c);
    }
  } catch (err) {
    _iterator.e(err);
  } finally {
    _iterator.f();
  }

  var _iterator2 = _createForOfIteratorHelper(allEdges),
      _step2;

  try {
    for (_iterator2.s(); !(_step2 = _iterator2.n()).done;) {
      var e = _step2.value;
      vDict[e.from].connect(vDict[e.to]);
    }
  } catch (err) {
    _iterator2.e(err);
  } finally {
    _iterator2.f();
  }

  var tarjan = new Tarjan(Object.values(vDict));
  tarjan.run();
  return [].concat(_toConsumableArray(graph), _toConsumableArray(edgesToAdd.filter(function (edge) {
    return vDict[edge.from].lowlink !== vDict[edge.to].lowlink;
  })));
};

var index = /*#__PURE__*/Object.freeze({
  __proto__: null,
  candidatesFromBallots: candidatesFromBallots,
  checkDuplicatedCandidate: checkDuplicatedCandidate,
  groupBallots: groupBallots,
  normalizeBallot: normalizeBallot,
  normalizeBallots: normalizeBallots,
  normalizeRankInput: normalizeRankInput,
  normalizeRanking: normalizeRanking,
  removeDuplicatedCandidates: removeDuplicatedCandidates,
  removeInvalidCandidates: removeInvalidCandidates,
  toWeightedBallots: toWeightedBallots,
  makeAntisymetric: makeAntisymetric,
  matrixFromBallots: matrixFromBallots,
  scoresToRanking: scoresToRanking,
  generateAcyclicGraph: generateAcyclicGraph
});

var scoresAny = function scoresAny(candidates, value) {
  return zipObject(candidates, new Array(candidates.length).fill(value));
};
var scoresZero = function scoresZero(candidates) {
  return scoresAny(candidates, 0);
};

var iterateFirstChoices = function iterateFirstChoices(ballots, candidates, computeBallotScore) {
  return iterateNthChoices(ballots, candidates, computeBallotScore, 0);
};
var iterateNthChoices = function iterateNthChoices(ballots, candidates, computeBallotScore, rankIdx) {
  var result = scoresZero(candidates);

  var _iterator = _createForOfIteratorHelper(normalizeBallots(ballots, candidates)),
      _step;

  try {
    for (_iterator.s(); !(_step = _iterator.n()).done;) {
      var ballot = _step.value;

      if (ballot.ranking.length > rankIdx) {
        var votes = ballot.ranking[rankIdx].filter(function (c) {
          return candidates.includes(c);
        });

        var _iterator2 = _createForOfIteratorHelper(votes),
            _step2;

        try {
          for (_iterator2.s(); !(_step2 = _iterator2.n()).done;) {
            var candidate = _step2.value;
            result[candidate] += computeBallotScore(votes, rankIdx) * ballot.weight;
          }
        } catch (err) {
          _iterator2.e(err);
        } finally {
          _iterator2.f();
        }
      }
    }
  } catch (err) {
    _iterator.e(err);
  } finally {
    _iterator.f();
  }

  return result;
};

var Method = /*#__PURE__*/function () {
  function Method(candidates) {
    _classCallCheck(this, Method);

    this.candidates = candidates;
  }

  _createClass(Method, [{
    key: "tieBreak",
    value: function tieBreak(rankingToTieBreak) {
      var ranking = this.ranking();
      var scores = ranking.reduce(function (acc, rank, idx) {
        return _objectSpread2(_objectSpread2({}, acc), _.zipObject(rank, new Array(rank.length).fill(-idx)));
      }, {});
      return rankingToTieBreak.flatMap(function (rank) {
        return scoresToRanking(_.pick(scores, rank));
      });
    }
  }]);

  return Method;
}();

var ScoreMethod = /*#__PURE__*/function (_Method) {
  _inherits(ScoreMethod, _Method);

  var _super = _createSuper(ScoreMethod);

  function ScoreMethod() {
    _classCallCheck(this, ScoreMethod);

    return _super.apply(this, arguments);
  }

  _createClass(ScoreMethod, [{
    key: "ranking",
    value: function ranking() {
      return scoresToRanking(this.scores());
    }
  }]);

  return ScoreMethod;
}(Method);

var BallotScoreMethod = /*#__PURE__*/function (_ScoreMethod) {
  _inherits(BallotScoreMethod, _ScoreMethod);

  var _super = _createSuper(BallotScoreMethod);

  function BallotScoreMethod(_ref) {
    var _this;

    var ballots = _ref.ballots,
        candidates = _ref.candidates;

    _classCallCheck(this, BallotScoreMethod);

    _this = _super.call(this, candidates);
    _this.ballots = ballots;
    return _this;
  }

  _createClass(BallotScoreMethod, [{
    key: "matrix",
    get: function get() {
      if (!this._matrix) this._matrix = matrixFromBallots(this.ballots, this.candidates);
      return this._matrix;
    }
  }]);

  return BallotScoreMethod;
}(ScoreMethod);

var Approbation = /*#__PURE__*/function (_BallotScoreMethod) {
  _inherits(Approbation, _BallotScoreMethod);

  var _super = _createSuper(Approbation);

  function Approbation() {
    _classCallCheck(this, Approbation);

    return _super.apply(this, arguments);
  }

  _createClass(Approbation, [{
    key: "scores",
    value: function scores() {
      return iterateFirstChoices(this.ballots, this.candidates, function () {
        return 1;
      });
    }
  }]);

  return Approbation;
}(BallotScoreMethod);

var computeScores$4 = function computeScores(candidates, ballots) {
  return iterateFirstChoices(ballots, candidates, function (rank) {
    return 1 / rank.length;
  });
};

var FirstPastThePost = /*#__PURE__*/function (_BallotScoreMethod) {
  _inherits(FirstPastThePost, _BallotScoreMethod);

  var _super = _createSuper(FirstPastThePost);

  function FirstPastThePost() {
    _classCallCheck(this, FirstPastThePost);

    return _super.apply(this, arguments);
  }

  _createClass(FirstPastThePost, [{
    key: "scores",
    value: function scores() {
      return computeScores$4(this.candidates, this.ballots);
    }
  }]);

  return FirstPastThePost;
}(BallotScoreMethod);

var AbsoluteMajority = /*#__PURE__*/function (_BallotScoreMethod) {
  _inherits(AbsoluteMajority, _BallotScoreMethod);

  var _super = _createSuper(AbsoluteMajority);

  function AbsoluteMajority() {
    _classCallCheck(this, AbsoluteMajority);

    return _super.apply(this, arguments);
  }

  _createClass(AbsoluteMajority, [{
    key: "scores",
    value: function scores() {
      var totalWeight = totalBallotsWeight(this.ballots);
      var fptp = new FirstPastThePost({
        ballots: this.ballots,
        candidates: this.candidates
      });
      var topRank = fptp.ranking()[0];
      var fptpScores = fptp.scores();
      if ((topRank === null || topRank === void 0 ? void 0 : topRank.length) === 1 && fptpScores[topRank[0]] > totalWeight / 2) return _objectSpread2(_objectSpread2({}, scoresZero(this.candidates)), {}, _defineProperty({}, topRank[0], fptpScores[topRank[0]]));
      return scoresZero(this.candidates);
    }
  }]);

  return AbsoluteMajority;
}(BallotScoreMethod);

var computeScores$3 = function computeScores(candidates, _ballots) {
  var scores = scoresZero(candidates);
  var ballots = normalizeBallots(_ballots, candidates);

  var _iterator = _createForOfIteratorHelper(normalizeBallots(ballots, candidates)),
      _step;

  try {
    for (_iterator.s(); !(_step = _iterator.n()).done;) {
      var ballot = _step.value;
      var voteValue = candidates.length; // - 1

      var _iterator2 = _createForOfIteratorHelper(ballot.ranking),
          _step2;

      try {
        for (_iterator2.s(); !(_step2 = _iterator2.n()).done;) {
          var candidatesAtRank = _step2.value;
          var value = voteValue - (candidatesAtRank.length - 1) / 2;

          var _iterator3 = _createForOfIteratorHelper(candidatesAtRank),
              _step3;

          try {
            for (_iterator3.s(); !(_step3 = _iterator3.n()).done;) {
              var candidate = _step3.value;
              scores[candidate] += value * ballot.weight;
            }
          } catch (err) {
            _iterator3.e(err);
          } finally {
            _iterator3.f();
          }

          voteValue -= candidatesAtRank.length;
        }
      } catch (err) {
        _iterator2.e(err);
      } finally {
        _iterator2.f();
      }
    }
  } catch (err) {
    _iterator.e(err);
  } finally {
    _iterator.f();
  }

  return scores;
};

var Borda = /*#__PURE__*/function (_BallotScoreMethod) {
  _inherits(Borda, _BallotScoreMethod);

  var _super = _createSuper(Borda);

  function Borda() {
    _classCallCheck(this, Borda);

    return _super.apply(this, arguments);
  }

  _createClass(Borda, [{
    key: "scores",
    value: function scores() {
      return computeScores$3(this.candidates, this.ballots);
    }
  }]);

  return Borda;
}(BallotScoreMethod);

var BallotMethod = /*#__PURE__*/function (_Method) {
  _inherits(BallotMethod, _Method);

  var _super = _createSuper(BallotMethod);

  function BallotMethod(_ref) {
    var _this;

    var ballots = _ref.ballots,
        candidates = _ref.candidates;

    _classCallCheck(this, BallotMethod);

    _this = _super.call(this, candidates);
    _this.ballots = ballots;
    return _this;
  }

  _createClass(BallotMethod, [{
    key: "matrix",
    get: function get() {
      if (!this._matrix) this._matrix = matrixFromBallots(this.ballots, this.candidates);
      return this._matrix;
    }
  }]);

  return BallotMethod;
}(Method);

var RoundBallotMethod = /*#__PURE__*/function (_BallotMethod) {
  _inherits(RoundBallotMethod, _BallotMethod);

  var _super = _createSuper(RoundBallotMethod);

  function RoundBallotMethod() {
    var _this;

    _classCallCheck(this, RoundBallotMethod);

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    _this = _super.call.apply(_super, [this].concat(args));

    _defineProperty(_assertThisInitialized(_this), "rounds", []);

    return _this;
  }

  _createClass(RoundBallotMethod, [{
    key: "computeRounds",
    value: function computeRounds() {
      while (this.lastRoundQualified().length > 0) {
        this.computeNextRound();
      }

      return this.rounds;
    }
  }, {
    key: "ranking",
    value: function ranking() {
      return _toConsumableArray(this.computeRounds()).reverse().map(function (r) {
        return r.roundResult.eliminated;
      }).filter(function (eliminated) {
        return eliminated.length;
      });
    }
  }, {
    key: "computeNextRound",
    value: function computeNextRound() {
      var previousRound = this.rounds.at(-1);
      var candidates = (previousRound === null || previousRound === void 0 ? void 0 : previousRound.roundResult.qualified) || this.candidates;
      var idx = previousRound ? previousRound.idx + 1 : 0;
      var round = {
        candidates: candidates,
        idx: idx,
        roundResult: this.round(candidates, idx)
      };
      this.rounds = [].concat(_toConsumableArray(this.rounds), [_objectSpread2({
        finished: round.roundResult.qualified.length === 0
      }, round)]);
    }
  }, {
    key: "lastRoundQualified",
    value: function lastRoundQualified() {
      var previousRound = this.rounds.at(-1);
      return (previousRound === null || previousRound === void 0 ? void 0 : previousRound.roundResult.qualified) || this.candidates;
    }
  }]);

  return RoundBallotMethod;
}(BallotMethod);

var _round$3 = function round(candidates, ballots) {
  var borda = new Borda({
    candidates: candidates,
    ballots: ballots
  });
  var ranking = borda.ranking();
  var eliminated = ranking.at(-1) || [];

  var qualified = _.difference(candidates, eliminated);

  return {
    eliminated: eliminated,
    qualified: qualified,
    scores: borda.scores()
  };
};

var Baldwin = /*#__PURE__*/function (_RoundBallotMethod) {
  _inherits(Baldwin, _RoundBallotMethod);

  var _super = _createSuper(Baldwin);

  function Baldwin() {
    _classCallCheck(this, Baldwin);

    return _super.apply(this, arguments);
  }

  _createClass(Baldwin, [{
    key: "round",
    value: function round(candidates) {
      return _round$3(candidates, this.ballots);
    }
  }]);

  return Baldwin;
}(RoundBallotMethod);

var MatrixScoreMethod = /*#__PURE__*/function (_ScoreMethod) {
  _inherits(MatrixScoreMethod, _ScoreMethod);

  var _super = _createSuper(MatrixScoreMethod);

  function MatrixScoreMethod(i) {
    var _this;

    _classCallCheck(this, MatrixScoreMethod);

    _this = _super.call(this, i.candidates);
    _this._matrix = {
      array: i.array,
      candidates: i.candidates
    };
    return _this;
  }

  _createClass(MatrixScoreMethod, [{
    key: "matrix",
    get: function get() {
      return this._matrix;
    }
  }]);

  return MatrixScoreMethod;
}(ScoreMethod);

var computeFromMatrix$2 = function computeFromMatrix(matrix) {
  var n = matrix.candidates.length;

  var p = _.range(n).map(function () {
    return _.range(n).map(function () {
      return 0;
    });
  });

  for (var i = 0; i < n; i++) {
    for (var j = 0; j < n; j++) {
      if (i !== j) {
        p[i][j] = (Math.sign(matrix.array[i][j] - matrix.array[j][i]) + 1) / 2;
      }
    }
  }

  var scores = p.map(function (m) {
    return m.reduce(function (acc, cur) {
      return acc + cur;
    }, 0);
  });
  return _.zipObject(matrix.candidates, scores);
};

var Copeland = /*#__PURE__*/function (_MatrixScoreMethod) {
  _inherits(Copeland, _MatrixScoreMethod);

  var _super = _createSuper(Copeland);

  function Copeland() {
    _classCallCheck(this, Copeland);

    return _super.apply(this, arguments);
  }

  _createClass(Copeland, [{
    key: "scores",
    value: function scores() {
      return computeFromMatrix$2(this.matrix);
    }
  }]);

  return Copeland;
}(MatrixScoreMethod);

var deTie = function deTie(ranking, ballots) {
  return ranking.flatMap(function (rank) {
    if (rank.length < 2) return [rank];
    var scores = iterateFirstChoices(ballots, rank, function (r) {
      return 1 / r.length;
    });

    var pickedScores = _.pick(scores, rank);

    var groups = _.groupBy(rank, function (c) {
      return pickedScores[c];
    });

    return Object.keys(groups).sort(function (a, b) {
      return Number(b) - Number(a);
    }).map(function (score) {
      return groups[score];
    });
  });
};

var isTied = function isTied(ranking) {
  return ranking.some(function (rank) {
    return rank.length > 1;
  });
};
var BiggestSupport = /*#__PURE__*/function (_BallotMethod) {
  _inherits(BiggestSupport, _BallotMethod);

  var _super = _createSuper(BiggestSupport);

  function BiggestSupport() {
    _classCallCheck(this, BiggestSupport);

    return _super.apply(this, arguments);
  }

  _createClass(BiggestSupport, [{
    key: "ranking",
    value: function ranking() {
      var ranking = [this.candidates];
      var rankIdx = 0;

      while (isTied(ranking) && rankIdx < this.candidates.length) {
        ranking = deTie(ranking, this.ballots);
        rankIdx++;
      }

      return ranking;
    }
  }]);

  return BiggestSupport;
}(BallotMethod);

var BottomTwoRunoff = /*#__PURE__*/function (_RoundBallotMethod) {
  _inherits(BottomTwoRunoff, _RoundBallotMethod);

  var _super = _createSuper(BottomTwoRunoff);

  function BottomTwoRunoff(i) {
    var _this;

    _classCallCheck(this, BottomTwoRunoff);

    _this = _super.call(this, i);
    _this.copeland = new Copeland(matrixFromBallots(i.ballots, i.candidates));
    return _this;
  }

  _createClass(BottomTwoRunoff, [{
    key: "round",
    value: function round(candidates) {
      var biggestSupport = new BiggestSupport({
        ballots: normalizeBallots(this.ballots, candidates),
        candidates: candidates
      }).ranking();
      var leastSupport = biggestSupport.at(-1);
      var bottom = [];
      if (leastSupport) bottom.push.apply(bottom, _toConsumableArray(leastSupport));

      if (bottom.length === 1) {
        var leastSupport2 = biggestSupport.at(-2);
        if (leastSupport2) bottom.push.apply(bottom, _toConsumableArray(leastSupport2));
      }

      var eliminated = this.copeland.tieBreak([bottom]).at(-1);
      if (!eliminated || eliminated.length === 0) throw new Error('Unexpected round result!');
      return {
        eliminated: eliminated,
        qualified: _.difference(candidates, eliminated),
        scores: _objectSpread2(_objectSpread2(_objectSpread2({}, scoresAny(candidates, 2)), scoresAny(bottom, 1)), scoresAny(eliminated, 0))
      };
    }
  }]);

  return BottomTwoRunoff;
}(RoundBallotMethod);

var reverseBallots = function reverseBallots(ballots) {
  return ballots.map(function (ballot) {
    return _objectSpread2(_objectSpread2({}, ballot), {}, {
      ranking: _toConsumableArray(ballot.ranking).reverse()
    });
  });
};

var _round$2 = function round(candidates, _ballots) {
  if (candidates.length < 2) return {
    eliminated: candidates,
    qualified: [],
    scores: scoresZero(candidates)
  };
  var ballots = normalizeBallots(_ballots, candidates);
  var absoluteMajority = new AbsoluteMajority({
    candidates: candidates,
    ballots: ballots
  });

  if (absoluteMajority.ranking()[0].length === 1) {
    var qualified = absoluteMajority.ranking()[0];
    return {
      eliminated: _.difference(candidates, qualified),
      qualified: qualified,
      scores: absoluteMajority.scores()
    };
  }

  var reversedFptp = new FirstPastThePost({
    ballots: reverseBallots(ballots),
    candidates: candidates
  });
  var eliminated = reversedFptp.ranking()[0];
  return {
    eliminated: eliminated,
    qualified: _.difference(candidates, eliminated),
    scores: _.mapValues(reversedFptp.scores(), function (s) {
      return -s;
    })
  };
};

var Coombs = /*#__PURE__*/function (_RoundBallotMethod) {
  _inherits(Coombs, _RoundBallotMethod);

  var _super = _createSuper(Coombs);

  function Coombs() {
    _classCallCheck(this, Coombs);

    return _super.apply(this, arguments);
  }

  _createClass(Coombs, [{
    key: "round",
    value: function round(candidates) {
      return _round$2(candidates, this.ballots);
    }
  }]);

  return Coombs;
}(RoundBallotMethod);

var _round$1 = function round(candidates, ballots) {
  var normalizedBallots = normalizeBallots(ballots, candidates);
  var fptp = new FirstPastThePost({
    ballots: normalizedBallots,
    candidates: candidates
  });
  var round1 = fptp.scores();
  var minScore = Math.min.apply(Math, _toConsumableArray(Object.values(round1)));
  var qualified = candidates.filter(function (c) {
    return round1[c] > minScore;
  });
  var eliminated = candidates.filter(function (c) {
    return round1[c] <= minScore;
  });
  return {
    eliminated: eliminated,
    qualified: qualified,
    scores: round1
  };
};

var InstantRunoff = /*#__PURE__*/function (_RoundBallotMethod) {
  _inherits(InstantRunoff, _RoundBallotMethod);

  var _super = _createSuper(InstantRunoff);

  function InstantRunoff() {
    _classCallCheck(this, InstantRunoff);

    return _super.apply(this, arguments);
  }

  _createClass(InstantRunoff, [{
    key: "round",
    value: function round(candidates) {
      return _round$1(candidates, this.ballots);
    }
  }]);

  return InstantRunoff;
}(RoundBallotMethod);

var rankingPenalty = function rankingPenalty(ranking, matrix) {
  var p = 0;

  for (var i = 0; i < matrix.length; i += 1) {
    for (var j = i + 1; j < matrix.length; j += 1) {
      p += matrix[ranking[i]][ranking[j]];
    }
  }

  return p;
};

var nextPermutation = function nextPermutation(arr) {
  // Find longest non-increasing suffix
  var array = _toConsumableArray(arr);

  var i = array.length - 1;

  while (i > 0 && array[i - 1] >= array[i]) {
    i -= 1;
  } // Now i is the head index of the suffix
  // Are we at the last permutation already?


  if (i <= 0) return false; // Let array[i - 1] be the pivot
  // Find rightmost element that exceeds the pivot

  var j = array.length - 1;

  while (array[j] <= array[i - 1]) {
    j -= 1;
  } // Now the value array[j] will become the new pivot
  // Assertion: j >= i
  // Swap the pivot with j


  var temp = array[i - 1];
  array[i - 1] = array[j];
  array[j] = temp; // Reverse the suffix

  j = array.length - 1;

  while (i < j) {
    temp = array[i];
    array[i] = array[j];
    array[j] = temp;
    i += 1;
    j -= 1;
  } // Successfully computed the next permutation


  return array;
};

var computeScores$2 = function computeScores(matrix) {
  var bestPermutations = [];
  var bestScore = Infinity;
  var p = range(matrix.candidates.length);

  while (p) {
    var s = rankingPenalty(p, matrix.array);
    if (s === bestScore) bestPermutations.push(p);else if (s < bestScore) {
      bestScore = s;
      bestPermutations = [p];
    }
    p = nextPermutation(p);
  }

  var sumIdx = matrix.candidates.map(function (c, cIdx) {
    return bestPermutations.map(function (perm) {
      return perm.indexOf(cIdx);
    }).reduce(function (a, v) {
      return a + v;
    }, 0);
  });
  return zipObject(matrix.candidates, sumIdx);
};

var Kemeny = /*#__PURE__*/function (_MatrixScoreMethod) {
  _inherits(Kemeny, _MatrixScoreMethod);

  var _super = _createSuper(Kemeny);

  function Kemeny() {
    _classCallCheck(this, Kemeny);

    return _super.apply(this, arguments);
  }

  _createClass(Kemeny, [{
    key: "scores",
    value: function scores() {
      return computeScores$2(this.matrix);
    }
  }]);

  return Kemeny;
}(MatrixScoreMethod);

function _arrayWithHoles(arr) {
  if (Array.isArray(arr)) return arr;
}

function _iterableToArrayLimit(arr, i) {
  var _i = arr == null ? null : typeof Symbol !== "undefined" && arr[Symbol.iterator] || arr["@@iterator"];

  if (_i == null) return;
  var _arr = [];
  var _n = true;
  var _d = false;

  var _s, _e;

  try {
    for (_i = _i.call(arr); !(_n = (_s = _i.next()).done); _n = true) {
      _arr.push(_s.value);

      if (i && _arr.length === i) break;
    }
  } catch (err) {
    _d = true;
    _e = err;
  } finally {
    try {
      if (!_n && _i["return"] != null) _i["return"]();
    } finally {
      if (_d) throw _e;
    }
  }

  return _arr;
}

function _nonIterableRest() {
  throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}

function _slicedToArray(arr, i) {
  return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest();
}

var makeJudgement = function makeJudgement(candidates, ballots) {
  var judgements = _.zipObject(candidates, _toConsumableArray(new Array(candidates.length)).map(function () {
    return [0, 0, 0, 0, 0, 0];
  }));

  var _iterator = _createForOfIteratorHelper(ballots),
      _step;

  try {
    for (_iterator.s(); !(_step = _iterator.n()).done;) {
      var ballot = _step.value;

      var _iterator2 = _createForOfIteratorHelper(ballot.ranking.entries()),
          _step2;

      try {
        for (_iterator2.s(); !(_step2 = _iterator2.n()).done;) {
          var _step2$value = _slicedToArray(_step2.value, 2),
              rankIdx = _step2$value[0],
              rank = _step2$value[1];

          var _iterator3 = _createForOfIteratorHelper(rank),
              _step3;

          try {
            for (_iterator3.s(); !(_step3 = _iterator3.n()).done;) {
              var can = _step3.value;
              if (candidates.includes(can)) judgements[can][rankIdx < 5 ? rankIdx : 5] += ballot.weight;
            }
          } catch (err) {
            _iterator3.e(err);
          } finally {
            _iterator3.f();
          }
        }
      } catch (err) {
        _iterator2.e(err);
      } finally {
        _iterator2.f();
      }
    }
  } catch (err) {
    _iterator.e(err);
  } finally {
    _iterator.f();
  }

  return judgements;
};

var getMedian = function getMedian(arr) {
  var sumWeights = _.sum(arr);

  var s = 0;
  var i = 0;
  var prevI = 0;
  var med = 0;

  var _iterator4 = _createForOfIteratorHelper(arr),
      _step4;

  try {
    for (_iterator4.s(); !(_step4 = _iterator4.n()).done;) {
      var j = _step4.value;
      s += j;
      i++;
      if (s === sumWeights / 2 + 1) med = (prevI + i) / 2;else if (s > sumWeights / 2) med = i;else {
        if (j > 0) prevI = i;
        continue;
      }
      break;
    }
  } catch (err) {
    _iterator4.e(err);
  } finally {
    _iterator4.f();
  }

  return med - 1;
};

var getMedians = function getMedians(judgements) {
  var candidates = Object.keys(judgements);
  var medians = {};

  for (var _i = 0, _candidates = candidates; _i < _candidates.length; _i++) {
    var c = _candidates[_i];
    medians[c] = getMedian(judgements[c]);
  }

  return medians;
};

var majorityJudgmentRanking = function majorityJudgmentRanking(candidates, ballots) {
  var judgements = makeJudgement(candidates, ballots);
  return tieBreak(judgements).reverse();
};

var tieBreak = function tieBreak(judgements) {
  var medians = getMedians(judgements);
  var ranking = scoresToRanking(medians);
  return ranking.flatMap(function (cs) {
    var median = medians[cs[0]];
    if (median === -1 || !Number.isInteger(median)) return [cs];

    var j = _.pick(judgements, cs);

    var minGroup = Math.min.apply(Math, _toConsumableArray(cs.map(function (c) {
      return j[c][median];
    })));
    if (minGroup <= 0) return [cs];

    var j2 = _.mapValues(j, function (jc) {
      return Object.assign([], jc, _defineProperty({}, median, jc[median] - minGroup));
    });

    return tieBreak(j2);
  });
};

var MajorityJudgment = /*#__PURE__*/function (_BallotMethod) {
  _inherits(MajorityJudgment, _BallotMethod);

  var _super = _createSuper(MajorityJudgment);

  function MajorityJudgment() {
    _classCallCheck(this, MajorityJudgment);

    return _super.apply(this, arguments);
  }

  _createClass(MajorityJudgment, [{
    key: "judgements",
    value: function judgements() {
      return makeJudgement(this.candidates, this.ballots);
    }
  }, {
    key: "medians",
    value: function medians() {
      return getMedians(this.judgements());
    }
  }, {
    key: "ranking",
    value: function ranking() {
      return majorityJudgmentRanking(this.candidates, this.ballots);
    }
  }]);

  return MajorityJudgment;
}(BallotMethod);

var simplexTableau = function simplexTableau(A) {
  var n = A.length;
  var extraZeros = new Array(n).fill(0);
  var obj = [0].concat(_toConsumableArray(new Array(n).fill(-1)), [1], _toConsumableArray(extraZeros), [0, 1]);
  var AnZeros = A.map(function (a, k) {
    return [0].concat(_toConsumableArray(a.map(function (v) {
      return v;
    })), [0], _toConsumableArray(extraZeros.map(function (v, kk) {
      return kk === k ? 1 : 0;
    })), [0, 0]);
  }); // set the boundary as x1 + x2 + ... + x(n-1) + 2 * xn < 1
  // cannot use the same function as the score function because

  var sum1 = [1].concat(_toConsumableArray(new Array(n).fill(1)), [0], _toConsumableArray(extraZeros), [1, 0]);
  return [obj].concat(_toConsumableArray(AnZeros), [sum1]);
};

var findPivotColumn = function findPivotColumn(T) {
  var minValue = Infinity;
  var minIndex = null;

  for (var k = 1; k < T[0].length; k++) {
    var v = T[0][k];

    if (v < 0 && v < minValue && k < T[0].length - 1) {
      minValue = v;
      minIndex = k;
    }
  }

  return minIndex;
};

var findPivotRow = function findPivotRow(T, col, usedRows) {
  var minRatio = Number.POSITIVE_INFINITY;
  var minIndex = null;

  for (var row = 1; row < T.length; row++) {
    if (T[row][col] !== 0 && !usedRows.includes(row)) {
      var ratio = T[row][0] / T[row][col];

      if (ratio < minRatio) {
        minRatio = ratio;
        minIndex = row;
      }
    }
  }

  return minIndex;
};
var pivot = function pivot(tableau, col, row) {
  var scaledRow = tableau[row].map(function (v) {
    return v / tableau[row][col];
  });
  return tableau.map(function (r, rowKey) {
    return rowKey === row ? scaledRow : r.map(function (value, colKey) {
      return value - r[col] * scaledRow[colKey];
    });
  });
};
var performPivots = function performPivots(tableau, usedRows) {
  var pivotedTable = tableau;
  var col = findPivotColumn(pivotedTable);

  while (col !== null) {
    var row = findPivotRow(pivotedTable, col, usedRows);
    if (!row) throw new Error('no row no pivot');
    usedRows.push(row);
    pivotedTable = pivot(pivotedTable, col, row);
    col = findPivotColumn(pivotedTable);
  }

  return pivotedTable;
};

var vectorFromPivotedTableau = function vectorFromPivotedTableau(pivotedTableau, size) {
  return new Array(size).fill(null).map(function (v, k) {
    if (pivotedTableau[0][k + 1] === 0) {
      var row = pivotedTableau.findIndex(function (r) {
        return r[k + 1] === 1;
      });
      if (row > -1) return pivotedTableau[row][0];
    }

    return 0;
  });
};

var solve = function solve(antisymetricMatrix) {
  var tableau = simplexTableau(antisymetricMatrix);
  var usedRows = [];
  var finalPivotedTable = performPivots(tableau, usedRows);
  return vectorFromPivotedTableau(finalPivotedTable, antisymetricMatrix.length);
};

var findCondorcet = function findCondorcet(_ref) {
  var candidates = _ref.candidates,
      array = _ref.array;
  var dominatingDirectList = candidates.map(function (c, k) {
    return array[k].map(function (v, k2) {
      return k2;
    }).filter(function (k2) {
      return array[k][k2] >= 0;
    });
  });
  var dominatingList = candidates.map(function (c, k) {
    var dominating = [k];
    var toCheck = [k];

    while (toCheck.length > 0) {
      var check = toCheck.pop();

      var toAdd = _.difference(dominatingDirectList[check], dominating);

      dominating = [].concat(_toConsumableArray(dominating), _toConsumableArray(toAdd));
      toCheck = [].concat(_toConsumableArray(toCheck), _toConsumableArray(toAdd));
    }

    return dominating.length === candidates.length;
  });
  return {
    array: array.filter(function (row, k1) {
      return dominatingList[k1];
    }).map(function (row) {
      return row.filter(function (v, k2) {
        return dominatingList[k2];
      });
    }),
    candidates: candidates.filter(function (c, k) {
      return dominatingList[k];
    })
  };
};

var RandomMethod = /*#__PURE__*/function (_Method) {
  _inherits(RandomMethod, _Method);

  var _super = _createSuper(RandomMethod);

  function RandomMethod(i) {
    var _this;

    _classCallCheck(this, RandomMethod);

    _this = _super.call(this, i.candidates);
    _this.rng = i.rng || Math.random;
    return _this;
  }

  return _createClass(RandomMethod);
}(Method);

var shuffleArray = function shuffleArray(_array, rng) {
  var array = _toConsumableArray(_array);

  var m = array.length;
  var t;
  var i; // While there remain elements to shuffle…

  while (m) {
    // Pick a remaining element…
    var r = rng();
    i = Math.floor(r * m--);
    t = array[m];
    array[m] = array[i];
    array[i] = t;
  }

  return array;
};

var randomRankingFromSores = function randomRankingFromSores(scoreObject, random) {
  var candidates = Object.keys(scoreObject);
  if (candidates.length < 2) return candidates;

  var sumScores = _.sum(Object.values(scoreObject));

  if (sumScores <= 0) return shuffleArray(candidates, random);

  var normalizedScoreObject = _.mapValues(scoreObject, function (s) {
    return s / sumScores;
  });

  var pickAt = random();
  var w = 0;

  for (var _i = 0, _candidates = candidates; _i < _candidates.length; _i++) {
    var candidate = _candidates[_i];
    w += normalizedScoreObject[candidate];
    if (w >= pickAt) return [candidate].concat(_toConsumableArray(randomRankingFromSores(_.omit(scoreObject, candidate), random)));
  }

  throw new Error('Unable to generate random ranking from scores');
};

var RandomMatrixMethod = /*#__PURE__*/function (_RandomMethod) {
  _inherits(RandomMatrixMethod, _RandomMethod);

  var _super = _createSuper(RandomMatrixMethod);

  function RandomMatrixMethod(i) {
    var _this;

    _classCallCheck(this, RandomMatrixMethod);

    _this = _super.call(this, i);
    _this._matrix = {
      array: i.array,
      candidates: i.candidates
    };
    return _this;
  }

  _createClass(RandomMatrixMethod, [{
    key: "matrix",
    get: function get() {
      return this._matrix;
    }
  }, {
    key: "ranking",
    value: function ranking() {
      return randomRankingFromSores(this.scores(), this.rng).map(function (c) {
        return [c];
      });
    }
  }]);

  return RandomMatrixMethod;
}(RandomMethod);

var computeLottery = function computeLottery(_matrix) {
  var matrix = makeAntisymetric(_matrix);
  var condorset = findCondorcet(matrix);
  var subVector = solve(condorset.array).map(function (v) {
    return Math.max(0, v);
  }); // Fixups because I can't implement the simplex algorithm correctly
  //  ---------- Begin ----------

  var sum = subVector.reduce(function (acc, cur) {
    return acc + cur;
  }, 0);
  var normalizedVector = subVector.map(function (v) {
    return v / sum;
  }); //  ----------  End  ----------

  return _objectSpread2(_objectSpread2({}, scoresZero(matrix.candidates)), zipObject(condorset.candidates, normalizedVector));
};
var MaximalLotteries = /*#__PURE__*/function (_RandomMatrixMethod) {
  _inherits(MaximalLotteries, _RandomMatrixMethod);

  var _super = _createSuper(MaximalLotteries);

  function MaximalLotteries() {
    _classCallCheck(this, MaximalLotteries);

    return _super.apply(this, arguments);
  }

  _createClass(MaximalLotteries, [{
    key: "scores",
    value: function scores() {
      return computeLottery(this.matrix);
    }
  }]);

  return MaximalLotteries;
}(RandomMatrixMethod);

var _scoreXY;
var MinimaxVariant;

(function (MinimaxVariant) {
  MinimaxVariant["WinningVotes"] = "WINNING_VOTES";
  MinimaxVariant["Margins"] = "MARGINS";
  MinimaxVariant["PairwiseOpposition"] = "PAIRWISE_OPPOSITION";
})(MinimaxVariant || (MinimaxVariant = {}));

var scoreXY = (_scoreXY = {}, _defineProperty(_scoreXY, MinimaxVariant.WinningVotes, function (xOverY, yOverX) {
  return xOverY > yOverX ? xOverY : 0;
}), _defineProperty(_scoreXY, MinimaxVariant.Margins, function (xOverY, yOverX) {
  return xOverY - yOverX;
}), _defineProperty(_scoreXY, MinimaxVariant.PairwiseOpposition, function (xOverY) {
  return xOverY;
}), _scoreXY);

var computeScores$1 = function computeScores(matrix, variant) {
  var s = {};

  var _iterator = _createForOfIteratorHelper(matrix.candidates.entries()),
      _step;

  try {
    var _loop = function _loop() {
      var _step$value = _slicedToArray(_step.value, 2),
          k = _step$value[0],
          c = _step$value[1];

      s[c] = -Math.max.apply(Math, _toConsumableArray(matrix.array[k].map(function (v, k2) {
        return scoreXY[variant](matrix.array[k2][k], v);
      })));
    };

    for (_iterator.s(); !(_step = _iterator.n()).done;) {
      _loop();
    }
  } catch (err) {
    _iterator.e(err);
  } finally {
    _iterator.f();
  }

  return s;
};

var Minimax = /*#__PURE__*/function (_MatrixScoreMethod) {
  _inherits(Minimax, _MatrixScoreMethod);

  var _super = _createSuper(Minimax);

  function Minimax(i) {
    var _this;

    _classCallCheck(this, Minimax);

    _this = _super.call(this, i);
    _this.minimaxVariant = i.variant || MinimaxVariant.Margins;
    return _this;
  }

  _createClass(Minimax, [{
    key: "scores",
    value: function scores() {
      return computeScores$1(this.matrix, this.minimaxVariant);
    }
  }]);

  return Minimax;
}(MatrixScoreMethod);

var _round = function round(candidates, ballots) {
  var borda = new Borda({
    candidates: candidates,
    ballots: ballots
  });
  var bordaScores = borda.scores();
  var scores = Object.values(bordaScores);
  var avg = sum(scores) / scores.length;
  var eliminated = candidates.filter(function (c) {
    return bordaScores[c] <= avg;
  });

  var qualified = _.difference(candidates, eliminated);

  return {
    eliminated: eliminated,
    qualified: qualified,
    scores: bordaScores
  };
};

var Nanson = /*#__PURE__*/function (_RoundBallotMethod) {
  _inherits(Nanson, _RoundBallotMethod);

  var _super = _createSuper(Nanson);

  function Nanson() {
    _classCallCheck(this, Nanson);

    return _super.apply(this, arguments);
  }

  _createClass(Nanson, [{
    key: "round",
    value: function round(candidates) {
      return _round(candidates, this.ballots);
    }
  }]);

  return Nanson;
}(RoundBallotMethod);

var computeFromMatrix$1 = function computeFromMatrix(matrix) {
  var allEdges = matrix.array.flatMap(function (row, from) {
    return row.map(function (value, to) {
      return value > 0 && to !== from ? {
        from: from,
        to: to,
        value: value
      } : null;
    }).filter(Boolean);
  });
  var edgesGroups = groupBy(allEdges, 'value');
  var groups = Object.keys(edgesGroups).sort(function (a, b) {
    return Number(b) - Number(a);
  }).map(function (value) {
    return edgesGroups[value];
  });
  var acyclicGraph = groups.reduce(function (graph, edgesToAdd) {
    return generateAcyclicGraph(graph, edgesToAdd);
  }, []);
  var graphsWinners = range(matrix.candidates.length).filter(function (candidate, key) {
    return !acyclicGraph.some(function (_ref) {
      var to = _ref.to;
      return to === key;
    });
  });
  var scores = graphsWinners.reduce(function (acc, curr) {
    acc[curr] = (acc[curr] || 0) + 1;
    return acc;
  }, {});
  var maxScore1 = Math.max.apply(Math, _toConsumableArray(Object.values(scores)));
  var winnersIdx = range(matrix.candidates.length).filter(function (i) {
    return scores[i] === maxScore1;
  });
  if (winnersIdx.length === matrix.candidates.length) return zipObject(matrix.candidates, new Array(matrix.candidates.length).fill(1));
  var nextResults = computeFromMatrix({
    array: matrix.array.filter(function (c, k) {
      return !winnersIdx.includes(k);
    }).map(function (row) {
      return row.filter(function (c, k) {
        return !winnersIdx.includes(k);
      });
    }),
    candidates: matrix.candidates.filter(function (c, k) {
      return !winnersIdx.includes(k);
    })
  });
  var maxScore2 = Math.max.apply(Math, _toConsumableArray(Object.values(nextResults)));
  return winnersIdx.reduce(function (scoreObject, winnerIdx) {
    return _objectSpread2(_objectSpread2({}, scoreObject), {}, _defineProperty({}, matrix.candidates[winnerIdx], maxScore2 + 1));
  }, nextResults);
};

var RankedPairs = /*#__PURE__*/function (_MatrixScoreMethod) {
  _inherits(RankedPairs, _MatrixScoreMethod);

  var _super = _createSuper(RankedPairs);

  function RankedPairs() {
    _classCallCheck(this, RankedPairs);

    return _super.apply(this, arguments);
  }

  _createClass(RankedPairs, [{
    key: "scores",
    value: function scores() {
      return computeFromMatrix$1(this.matrix);
    }
  }]);

  return RankedPairs;
}(MatrixScoreMethod);

var RandomCandidates = /*#__PURE__*/function (_RandomMethod) {
  _inherits(RandomCandidates, _RandomMethod);

  var _super = _createSuper(RandomCandidates);

  function RandomCandidates() {
    _classCallCheck(this, RandomCandidates);

    return _super.apply(this, arguments);
  }

  _createClass(RandomCandidates, [{
    key: "ranking",
    value: function ranking() {
      return shuffleArray(this.candidates, this.rng).map(function (c) {
        return [c];
      });
    }
  }, {
    key: "scores",
    value: function scores() {
      return zipObject(this.candidates, new Array(this.candidates.length).fill(1 / this.candidates.length));
    }
  }]);

  return RandomCandidates;
}(RandomMethod);

var RandomBallotMethod = /*#__PURE__*/function (_BallotMethod) {
  _inherits(RandomBallotMethod, _BallotMethod);

  var _super = _createSuper(RandomBallotMethod);

  function RandomBallotMethod(i) {
    var _this;

    _classCallCheck(this, RandomBallotMethod);

    _this = _super.call(this, i);
    _this.rng = i.rng || Math.random;
    return _this;
  }

  return _createClass(RandomBallotMethod);
}(BallotMethod);

var pickBallotIdx = function pickBallotIdx(ballots, ratio) {
  var W = totalBallotsWeight(ballots);
  var pickAt = ratio * W;
  var w = 0;
  var i = 0;

  while (w < pickAt) {
    var b = ballots[i];
    if (!b) throw new Error('Could not pick a ballot?!');
    w += b.weight;
    if (w >= pickAt) return i;
    i++;
  }

  throw new Error('Could not pick a ballot?!');
};

var rank = function rank(candidates, ballots, rng) {
  if (candidates.length === 0) return [];
  if (ballots.length === 0) return [candidates];
  var ratio = rng();
  var idx = pickBallotIdx(ballots, ratio);
  return ballots[idx].ranking;
};

var RandomDictator = /*#__PURE__*/function (_RandomBallotMethod) {
  _inherits(RandomDictator, _RandomBallotMethod);

  var _super = _createSuper(RandomDictator);

  function RandomDictator(i) {
    _classCallCheck(this, RandomDictator);

    return _super.call(this, i);
  }

  _createClass(RandomDictator, [{
    key: "ranking",
    value: function ranking() {
      return rank(this.candidates, this.ballots, this.rng);
    }
  }]);

  return RandomDictator;
}(RandomBallotMethod);

var computeScores = function computeScores(matrix) {
  var antisymetric = makeAntisymetric(matrix);

  var antisymetricUnit = _objectSpread2(_objectSpread2({}, antisymetric), {}, {
    array: antisymetric.array.map(function (r) {
      return r.map(function (v) {
        return Math.sign(v);
      });
    })
  });

  return computeLottery(antisymetricUnit);
};

var RandomizedCondorcet = /*#__PURE__*/function (_RandomMatrixMethod) {
  _inherits(RandomizedCondorcet, _RandomMatrixMethod);

  var _super = _createSuper(RandomizedCondorcet);

  function RandomizedCondorcet() {
    _classCallCheck(this, RandomizedCondorcet);

    return _super.apply(this, arguments);
  }

  _createClass(RandomizedCondorcet, [{
    key: "scores",
    value: function scores() {
      return computeScores(this.matrix);
    }
  }]);

  return RandomizedCondorcet;
}(RandomMatrixMethod);

var computeFromMatrix = function computeFromMatrix(matrix) {
  var n = matrix.candidates.length;
  var p = range(n).map(function () {
    return range(n).map(function () {
      return 0;
    });
  });

  for (var i = 0; i < n; i++) {
    for (var j = 0; j < n; j++) {
      if (i !== j) if (matrix.array[i][j] > matrix.array[j][i]) p[i][j] = matrix.array[i][j];else p[i][j] = 0;
    }
  }

  for (var _i = 0; _i < n; _i++) {
    for (var _j = 0; _j < n; _j++) {
      if (_i !== _j) for (var k = 0; k < n; k++) {
        if (_i !== k && _j !== k) p[_j][k] = Math.max(p[_j][k], Math.min(p[_j][_i], p[_i][k]));
      }
    }
  }

  var s = {};

  var _iterator = _createForOfIteratorHelper(matrix.candidates.entries()),
      _step;

  try {
    var _loop = function _loop() {
      var _step$value = _slicedToArray(_step.value, 2),
          k = _step$value[0],
          c = _step$value[1];

      s[c] = p[k].filter(function (v, k2) {
        return v > p[k2][k];
      }).length;
    };

    for (_iterator.s(); !(_step = _iterator.n()).done;) {
      _loop();
    }
  } catch (err) {
    _iterator.e(err);
  } finally {
    _iterator.f();
  }

  return s;
};

var Schulze = /*#__PURE__*/function (_MatrixScoreMethod) {
  _inherits(Schulze, _MatrixScoreMethod);

  var _super = _createSuper(Schulze);

  function Schulze() {
    _classCallCheck(this, Schulze);

    return _super.apply(this, arguments);
  }

  _createClass(Schulze, [{
    key: "scores",
    value: function scores() {
      return computeFromMatrix(this.matrix);
    }
  }]);

  return Schulze;
}(MatrixScoreMethod);

var TwoRoundRunoff = /*#__PURE__*/function (_RoundBallotMethod) {
  _inherits(TwoRoundRunoff, _RoundBallotMethod);

  var _super = _createSuper(TwoRoundRunoff);

  function TwoRoundRunoff() {
    _classCallCheck(this, TwoRoundRunoff);

    return _super.apply(this, arguments);
  }

  _createClass(TwoRoundRunoff, [{
    key: "round",
    value: function round(candidates, idx) {
      var scores = new FirstPastThePost({
        ballots: this.ballots,
        candidates: candidates
      }).scores();
      var absoluteMajority = new AbsoluteMajority({
        ballots: this.ballots,
        candidates: candidates
      });

      if (absoluteMajority.ranking().length > 1) {
        var qualified = absoluteMajority.ranking().at(0);
        if (!qualified) throw new Error('Unexpected error in absolute majority computation');
        return {
          eliminated: _.difference(candidates, qualified),
          qualified: qualified,
          scores: scores
        };
      }

      var scoreValues = Object.values(scores).sort(function (a, b) {
        return b - a;
      });

      if (idx === 0) {
        var _qualified = candidates.filter(function (c) {
          return scores[c] >= scoreValues[1];
        });

        return {
          eliminated: _.difference(candidates, _qualified),
          qualified: _qualified,
          scores: scores
        };
      }

      if (idx === 1) {
        var _qualified2 = candidates.filter(function (c) {
          return scores[c] >= scoreValues[0];
        });

        return {
          eliminated: _.difference(candidates, _qualified2),
          qualified: _qualified2,
          scores: scores
        };
      }

      return {
        eliminated: candidates,
        qualified: [],
        scores: scores
      };
    }
  }]);

  return TwoRoundRunoff;
}(RoundBallotMethod);

var _methods;
var methods = (_methods = {}, _defineProperty(_methods, VotingSystem.AbsoluteMajority, AbsoluteMajority), _defineProperty(_methods, VotingSystem.Approbation, Approbation), _defineProperty(_methods, VotingSystem.Baldwin, Baldwin), _defineProperty(_methods, VotingSystem.Borda, Borda), _defineProperty(_methods, VotingSystem.BottomTwoRunoff, BottomTwoRunoff), _defineProperty(_methods, VotingSystem.Coombs, Coombs), _defineProperty(_methods, VotingSystem.Copeland, Copeland), _defineProperty(_methods, VotingSystem.FirstPastThePost, FirstPastThePost), _defineProperty(_methods, VotingSystem.InstantRunoff, InstantRunoff), _defineProperty(_methods, VotingSystem.Kemeny, Kemeny), _defineProperty(_methods, VotingSystem.MajorityJudgment, MajorityJudgment), _defineProperty(_methods, VotingSystem.MaximalLotteries, MaximalLotteries), _defineProperty(_methods, VotingSystem.Minimax, Minimax), _defineProperty(_methods, VotingSystem.Nanson, Nanson), _defineProperty(_methods, VotingSystem.RandomCandidates, RandomCandidates), _defineProperty(_methods, VotingSystem.RandomDictator, RandomDictator), _defineProperty(_methods, VotingSystem.RandomizedCondorcet, RandomizedCondorcet), _defineProperty(_methods, VotingSystem.RankedPairs, RankedPairs), _defineProperty(_methods, VotingSystem.Schulze, Schulze), _defineProperty(_methods, VotingSystem.TwoRoundRunoff, TwoRoundRunoff), _methods);

export { AbsoluteMajority, Approbation, Baldwin, Borda, BottomTwoRunoff, Coombs, Copeland, FirstPastThePost, InstantRunoff, Kemeny, MajorityJudgment, MaximalLotteries, Minimax, Nanson, RandomCandidates, RandomDictator, RandomizedCondorcet, RankedPairs, Schulze, TwoRoundRunoff, VotingSystem, methods, index as utils };
